/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.http.listeners;

import java.util.Properties;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange;

import org.objectweb.petals.binding.http.HttpBCImpl;
import org.objectweb.petals.component.framework.PEtALSCDKException;
import org.objectweb.petals.component.framework.util.Exchange;
import org.objectweb.petals.component.framework.util.Extensions;
import org.objectweb.petals.component.framework.util.StringHelper;

/**
 * This class is used to build a session descriptor from service unit descriptor
 * @author mcarpentier - EBM Websourcing
 */
public class HttpRequestDescriptorBuilder {

    /**
     * Logger
     */
    private Logger log;

    /**
     * @param log
     *            the component logger
     */
    public HttpRequestDescriptorBuilder(final Logger log) {
        this.log = log;
    }

    /**
     * Retrieve http request properties from the service unit descriptor
     * Descriptor default values : <br />
     * method : GET<br />
     * @param extensions
     *            extensions associated to the given message exchange
     * @param exchange
     *            the {@link MessageExchange} containing the text content and
     *            the optionnal parameters
     * @return a <b>{@link HttpRequestDescriptor}</b> filled in with extracted
     *         information
     * @throws PEtALSCFException
     *             if properties checking fails
     */
    public final HttpRequestDescriptor build(final Extensions extensions,
            final Exchange exchange) throws PEtALSCDKException {
        final HttpRequestDescriptor requestDescriptor = new HttpRequestDescriptor();
        final Properties extensionsProps = extensions.getPetalsExtensions();

        // Retrieve user and password
        final String url = extensionsProps
                .getProperty(HttpBCImpl.URL_PATHELEMENT);
        final String method = extensionsProps.getProperty(
                HttpBCImpl.METHOD_PATHELEMENT, HttpBCImpl.HTTP_GET_METHOD);

        // add all properties to the message exchange
        for (Object name : exchange.getPropertyNames()) {
            final String nameString = (String) name;
            if ((!(nameString.startsWith(HttpBCImpl.JBI_NS)))
                    && (!(nameString.startsWith(HttpBCImpl.PETALS_NS)))) {
                if (HttpBCImpl.SUFFIX_PROPERTY.equalsIgnoreCase(nameString)) {
                    requestDescriptor.setSuffix((String) exchange
                            .getProperty(nameString));
                } else {
                    requestDescriptor.addParameter(nameString,
                            (String) exchange.getProperty(nameString));
                }
            }
        }

        requestDescriptor.setMethod(method);
        requestDescriptor.setUrl(url);

        // Check mandatory properties
        this.checkProperties(requestDescriptor);

        return requestDescriptor;
    }

    /**
     * Check the properties extracted from the service unit descriptor. URL
     * property is mandatory
     * @param requestDescriptor
     *            the current {@link HttpRequestDescriptor}
     * @throws PEtALSCFException
     *             if a checking step failed
     */
    protected final void checkProperties(final HttpRequestDescriptor requestDescriptor)
        throws PEtALSCDKException {
        String msg = "";
        if (StringHelper.isNullOrEmpty(requestDescriptor.getUrl())) {
            msg = "The URL must be specified";
            this.log.severe(msg);
            throw new PEtALSCDKException(msg);
        }
    }

    /**
     * Build the full URL with all registered parameters
     * @param requestDescriptor
     *            the current {@link HttpRequestDescriptor}
     * @return the full URL
     */
    public final String buildRequestUrl(final HttpRequestDescriptor requestDescriptor) {
        String url = requestDescriptor.getUrl();

        if (!(StringHelper.isNullOrEmpty(requestDescriptor.getSuffix()))) {
            if (url.endsWith("/")) {
                url = url + requestDescriptor.getSuffix();
            } else {
                url = url + "/" + requestDescriptor.getSuffix();
            }
        }

        if (url.endsWith("/")) {
            url = url.substring(0, url.length() - 1);
        }

        if (requestDescriptor.getParameters() != null) {
            url = url + "?";
            int i = 0;

            for (String parameter : requestDescriptor.getParameters().keySet()) {
                if (i == 0) {
                    url = url + parameter + "="
                            + requestDescriptor.getParameters().get(parameter);
                } else {
                    url = url + "&" + parameter + "="
                            + requestDescriptor.getParameters().get(parameter);
                }
                i++;
            }

        }

        return url;

    }
}
